@@ -1,7 +1,7 @@
 var results = {
 
 "Acme-BOPE-0.01": [
-  {status:"PASS",id:"2966429",perl:"5.8.8",osname:"linux",ostext:"Linux",osvers:"2.6.16.60-0.31-default",archname:"s390x-linux",perlmat:"rel"}
+  {status:"PASS",id:"2966429",perl:"5.8.8",osname:"linux",ostext:"LINUX",osvers:"2.6.16.60-0.31-default",archname:"s390x-linux",perlmat:"rel"}
 ]
 };
 
